@@ -1,14 +1,14 @@
 from __future__ import annotations
 
-import typing as t
 import functools
 import itertools
+import typing as t
 from pathlib import Path
 
 import typer
 
+from pipelime.cli.parser import CLIParsingError, parse_pipelime_cli
 from pipelime.cli.subcommands import SubCommands as subc
-from pipelime.cli.parser import parse_pipelime_cli, CLIParsingError
 from pipelime.cli.utils import (
     PipelimeSymbolsHelper,
     print_command_op_stage_info,
@@ -92,8 +92,8 @@ def _process_cfg_or_die(
     verbose: bool,
     print_all: bool,
 ) -> t.List["XConfig"]:
-    from pipelime.cli.pretty_print import print_error, print_info
     from pipelime.choixe.visitors.processor import ChoixeProcessingError
+    from pipelime.cli.pretty_print import print_error, print_info, print_warning
 
     if verbose:
         print_info(f"> Processing {cfg_name}...")
@@ -111,6 +111,12 @@ def _process_cfg_or_die(
             )
             raise typer.Exit(1)
         raise e
+    except RecursionError:
+        print_error(f"Recursion detected while processing {cfg_name}!")
+        print_warning(
+            "Please check your context for self-references, eg, `myvar: $var(myvar)`."
+        )
+        raise typer.Exit(1)
 
     if verbose:
         pls = "s" if len(effective_configs) != 1 else ""
@@ -148,8 +154,8 @@ def _process_all(
     exit_on_error: bool,
     verbose: int,
 ):
-    from pipelime.cli.pretty_print import print_info
     from pipelime.choixe import XConfig
+    from pipelime.cli.pretty_print import print_info
 
     if verbose > 1:
         print_info("\nProcessing configurations:")
@@ -231,7 +237,7 @@ def pl_main(
             "all the arguments required by the command.\n\n"
             "`++opt` or `+opt` command line options update and override them.\n\n "
         ),
-        autocompletion=_complete_yaml,
+        # autocompletion=_complete_yaml,
     ),
     context: t.List[Path] = typer.Option(
         None,
@@ -252,7 +258,13 @@ def pl_main(
             "`@@opt` or `@opt` command line options update and override them.\n\n"
             "After a `//` token, `++opt` and `+opt` are accepted as well.\n\n "
         ),
-        autocompletion=_complete_yaml,
+        # autocompletion=_complete_yaml,
+    ),
+    keep_tmp: bool = typer.Option(
+        False,
+        "--keep-tmp",
+        "-t",
+        help="DO NOT remove temporary folders, if any, after running this command.",
     ),
     ctx_autoload: bool = typer.Option(
         True,
@@ -286,6 +298,12 @@ def pl_main(
         resolve_path=True,
         help="Save final processed context to json/yaml.",
     ),
+    command_outputs: t.Optional[Path] = typer.Option(
+        None,
+        writable=True,
+        resolve_path=True,
+        help="Save command outputs to json/yaml.",
+    ),
     verbose: int = typer.Option(
         0,
         "--verbose",
@@ -305,12 +323,12 @@ def pl_main(
             )
             + subc.get_help()
         ),
-        autocompletion=PipelimeSymbolsHelper.complete_name(
-            is_cmd=True,
-            is_seq_op=False,
-            is_stage=False,
-            additional_names=subc.get_autocompletions(),
-        ),
+        # autocompletion=PipelimeSymbolsHelper.complete_name(
+        #     is_cmd=True,
+        #     is_seq_op=False,
+        #     is_stage=False,
+        #     additional_names=subc.get_autocompletions(),
+        # ),
     ),
     command_args: t.Optional[t.List[str]] = typer.Argument(
         None,
@@ -544,6 +562,8 @@ def pl_main(
             effective_ctx = functools.reduce(
                 lambda acc, curr: _deep_update_fn(acc, curr, verbose > 3), effective_ctx
             )
+            if output_ctx:
+                effective_ctx.save_to(output_ctx)
         else:
             effective_ctx = XConfig()
 
@@ -648,7 +668,9 @@ def pl_main(
                     cmd_name = next(iter(cfg_dict))
                     cfg_dict = next(iter(cfg_dict.values()))
 
-                run_command(cmd_name, cfg_dict, verbose, dry_run)
+                run_command(
+                    cmd_name, cfg_dict, verbose, dry_run, keep_tmp, command_outputs
+                )
     else:
         from pipelime.cli.pretty_print import print_error
 
@@ -656,22 +678,41 @@ def pl_main(
         raise typer.Exit(1)
 
 
-def run_command(command: str, cmd_args: t.Mapping, verbose: int, dry_run: bool):
+def run_command(
+    command: str,
+    cmd_args: t.Mapping,
+    verbose: int,
+    dry_run: bool,
+    keep_tmp: bool,
+    command_outputs: t.Optional[Path],
+):
     """
     Run a pipelime command.
     """
 
     import time
+
     from pydantic.error_wrappers import ValidationError
 
-    from pipelime.cli.pretty_print import print_info, print_command_outputs
-    from pipelime.cli.utils import get_pipelime_command_cls, time_to_str, show_field_alias_valerr
+    from pipelime.choixe.utils.io import PipelimeTmp, dump
+    from pipelime.cli.pretty_print import print_command_outputs, print_info
+    from pipelime.cli.tui import TuiApp, is_tui_needed
+    from pipelime.cli.utils import (
+        get_pipelime_command_cls,
+        show_field_alias_valerr,
+        time_to_str,
+    )
+    from pipelime.commands import TempCommand
 
     try:
         cmd_cls = get_pipelime_command_cls(command)
     except ValueError:
         raise typer.Exit(1)
 
+    if is_tui_needed(cmd_cls, cmd_args):
+        app = TuiApp(cmd_cls, cmd_args)
+        cmd_args = t.cast(t.Mapping, app.run())
+
     if verbose > 2:
         print_info(f"\nCreating command `{command}` with options:")
         print_info(cmd_args, pretty=True)
@@ -698,6 +739,29 @@ def run_command(command: str, cmd_args: t.Mapping, verbose: int, dry_run: bool):
     print_info(f"\n`{command}` outputs:")
     print_command_outputs(cmd_obj)
 
+    if command_outputs:
+
+        def _cvt_data(data):
+            if hasattr(data, "__piper_repr__"):
+                return data.__piper_repr__()
+            if isinstance(data, (bytes, str)):
+                return str(data)
+            if isinstance(data, Path):
+                return data.resolve().absolute().as_posix()
+            elif isinstance(data, t.Mapping):
+                return {k: _cvt_data(v) for k, v in data.items()}
+            elif isinstance(data, t.Sequence):
+                return [_cvt_data(v) for v in data]
+            return repr(data)
+
+        print_info(f"\nSaving command outputs to `{command_outputs}`")
+        outs = {k: _cvt_data(v) for k, v in cmd_obj.get_outputs().items()}
+        dump(outs, command_outputs)
+
+    if not keep_tmp and PipelimeTmp.SESSION_TMP_DIR:
+        print_info("\nCleaning temporary files...")
+        TempCommand(name=PipelimeTmp.SESSION_TMP_DIR.stem, force=True)()  # type: ignore
+
 
 def _create_typer_app(
     *,
