@@ -218,23 +218,25 @@ class SamplesSequence(
             sample_fn (t.Optional[t.Callable[[Sample], None]], optional): a callable to
                 run on each sample  (Default to None)
             track_fn (track_fn: t.Union[bool, str, t.Callable[
-                [t.Iterable], t.Iterable], None], optional): if True, a rich trackbar
+                [t.Iterable], t.Iterable], None], optional): if True, a trackbar
                 is shown; if a string is passed, it is set as the message for the
-                default rich trackbar; otherwise you should provide your own callable
-                to track the progress, defaults to True  (Default to True)
+                default trackbar; otherwise you should provide your own callable
+                to track the progress  (Default to True)
         """
         from pipelime.sequences import Grabber, grab_all
 
         if isinstance(track_fn, (bool, str)):
-            import rich.progress
-
             if track_fn:
-                msg = track_fn if isinstance(track_fn, str) else ""
-                track_fn = lambda x: rich.progress.track(
-                    x,
-                    total=len(self),
-                    description=msg,
-                )
+                from pipelime.piper.progress.tracker.base import TqdmTask
+
+                message = track_fn if isinstance(track_fn, str) else ""
+
+                def _tqdm_track_fn(x):
+                    return TqdmTask.default_bar(
+                        iterable=x, total=len(self), message=message
+                    )
+
+                track_fn = _tqdm_track_fn
             else:
                 track_fn = None
 
@@ -347,6 +349,21 @@ class SamplesSequence(
         """
         ...
 
+    @samples_sequence_stub
+    @staticmethod
+    def from_images(
+        folder: "Path",  # type: ignore # noqa: E602,F821
+        *,
+        must_exist: bool = True,
+        image_key: str = "image",
+        sort_files: bool = True,
+        recursive: bool = True,
+    ) -> SamplesSequence:
+        """A SamplesSequence loading images from a folder.
+        Run `pipelime help from_images` to read the complete documentation.
+        """
+        ...
+
     @samples_sequence_stub
     @staticmethod
     def toy_dataset(
